import { R as RemoteUploaderService } from './el-result-s9MlQSRt.js';
import { i as isSafeInteger, a as ContentScriptToBackgroundEvent, b as init_1, f as firstValueFrom, M as MessagePortSubject, c as MAGIC, d as CommonEvent } from './message-CyATHsMQ.js';
import { g as getSessionStorage, r as recordStop, w as waitForClientConnect } from './fetch-w4DphyvP.js';
import { b as browser } from './browser-polyfill-CQCiMlvl.js';
import { r as removeTabAvailable, s as setTabAvailable } from './tabs-CVi7ISFn.js';
import { s as setupWebExtensionServer } from './server-EI8KcL4n.js';
import './index-Cma3Snfr.js';
import { b as bridge } from './content-script-D8k8WEo0.js';
import './vue-i18n.runtime-dWV3JVIG.js';
import './interval-2DfobCnA.js';
import './switchMap-2-0PRSXl.js';
import './chunk-E2HJRHOS-UwhYmLEN.js';
import './chunk-QIZ4XBKF-DinpiVrS.js';

async function setupIntermediary(bridge) {
    if (!window.parent || window.parent === window) {
        return;
    }
    const { sendMessage } = bridge;
    const tab = await browser.tabs.getCurrent();
    const browserTabId = tab?.id;
    if (!isSafeInteger(browserTabId)) {
        throw new Error();
    }
    const tabId = String(browserTabId);
    const { frameId } = await sendMessage(ContentScriptToBackgroundEvent.GetFrameId, null, 'background');
    const createId = init_1({
        fingerprint: String(frameId)
    });
    const session = getSessionStorage();
    const { onClientConnect, ctx } = setupWebExtensionServer(tabId, {
        session,
        async startRecord (args) {
            const { windowId } = tab;
            if (!windowId) {
                return;
            }
            return await sendMessage(CommonEvent.StartRecord, {
                ...args,
                tabId: browserTabId,
                windowId
            });
        }
    });
    const { recorderState$, container } = ctx;
    RemoteUploaderService.register(container, bridge);
    const isRootFrame = window.top === window.parent;
    if (isRootFrame && recorderState$.getValue().snapshotTabId === String(tabId)) {
        recordStop(recorderState$, false);
    }
    firstValueFrom(waitForClientConnect()).then((event)=>{
        const port = event.ports[0];
        const message$ = new MessagePortSubject(port, String(tab.id));
        onClientConnect(createId(), message$);
        if (isRootFrame) {
            window.onbeforeunload = ()=>{
                removeTabAvailable(browserTabId, String(frameId));
                if (recorderState$.getValue().snapshotTabId === String(tabId)) {
                    recordStop(recorderState$, false);
                }
            };
            return setTabAvailable(browserTabId, String(frameId));
        }
        return undefined;
    });
    window.parent.postMessage(MAGIC, '*');
}

setupIntermediary(bridge);
