import { R as REGION, o as ACTIVE_TAB_KEY, l as fromEvent, t as taskStatusBroadcast, Z as map, j as ENV_KEY } from './message-CyATHsMQ.js';
import { a as getSharedStorage, g as getSessionStorage, s as setupWebExtensionOAuth, i as createFetch, K as setupServer, O as OAUTH_CLIENT, M as OAUTH_CONTROLLER, p as syncRecorderState, v as syncStorage, n as storageObservable, N as RecorderClientTransport } from './fetch-w4DphyvP.js';
import './browser-polyfill-CQCiMlvl.js';
import { L as LoggerService } from './el-result-s9MlQSRt.js';

function setupWebExtensionServer(tabId, init) {
    const storage = getSharedStorage();
    const session = getSessionStorage();
    const { oauth, oauthClient } = setupWebExtensionOAuth(storage);
    const env$ = storageObservable(storage, ENV_KEY);
    const $fetch = createFetch(storage, oauth, REGION);
    const logger = new LoggerService(oauth, storage, REGION);
    env$.subscribe((env)=>{
        logger.setEnv(env || 'production');
    });
    const { ctx } = setupServer('browser_extension', storage, tabId, logger, $fetch, REGION);
    ctx.container.register(OAUTH_CLIENT, {
        useValue: oauthClient
    }).register(OAUTH_CONTROLLER, {
        useValue: oauth
    });
    const ready = Promise.all([
        syncRecorderState(ctx.recorderState$),
        syncStorage(ctx.activeTab$, session, ACTIVE_TAB_KEY, null)
    ]);
    function onClientConnect(documentId, message$) {
        const client = new RecorderClientTransport(message$, {
            ...ctx,
            ...init,
            tabId
        });
        client.onConnectAck(documentId);
        return client;
    }
    fromEvent(taskStatusBroadcast, 'message').pipe(map((event)=>event.data)).subscribe(ctx.taskStatusChange$);
    return {
        ctx,
        onClientConnect,
        ready,
        oauth,
        logger
    };
}

export { setupWebExtensionServer as s };
